'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var manageSideEffects = _interopDefault(require('manage-side-effects'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function noop() {}

function assign(tar, src) {
  for (var k in src) {
    tar[k] = src[k];
  }

  return tar;
}

function assignTrue(tar, src) {
  for (var k in src) {
    tar[k] = 1;
  }

  return tar;
}

function append(target, node) {
  target.appendChild(node);
}

function insert(target, node, anchor) {
  target.insertBefore(node, anchor);
}

function detachNode(node) {
  node.parentNode.removeChild(node);
}

function createElement(name) {
  return document.createElement(name);
}

function createText(data) {
  return document.createTextNode(data);
}

function createComment() {
  return document.createComment('');
}

function addListener(node, event, handler, options) {
  node.addEventListener(event, handler, options);
}

function removeListener(node, event, handler, options) {
  node.removeEventListener(event, handler, options);
}

function setAttribute(node, attribute, value) {
  if (value == null) node.removeAttribute(attribute);else node.setAttribute(attribute, value);
}

function toggleClass(element, name, toggle) {
  element.classList[toggle ? 'add' : 'remove'](name);
}

function destroyBlock(block, lookup) {
  block.d(1);
  lookup[block.key] = null;
}

function updateKeyedEach(old_blocks, component, changed, get_key, dynamic, ctx, list, lookup, node, destroy, create_each_block, intro_method, next, get_context) {
  var o = old_blocks.length;
  var n = list.length;
  var i = o;
  var old_indexes = {};

  while (i--) {
    old_indexes[old_blocks[i].key] = i;
  }

  var new_blocks = [];
  var new_lookup = {};
  var deltas = {};
  var i = n;

  while (i--) {
    var child_ctx = get_context(ctx, list, i);
    var key = get_key(child_ctx);
    var block = lookup[key];

    if (!block) {
      block = create_each_block(component, key, child_ctx);
      block.c();
    } else if (dynamic) {
      block.p(changed, child_ctx);
    }

    new_blocks[i] = new_lookup[key] = block;
    if (key in old_indexes) deltas[key] = Math.abs(i - old_indexes[key]);
  }

  var will_move = {};
  var did_move = {};

  function insert(block) {
    block[intro_method](node, next);
    lookup[block.key] = block;
    next = block.first;
    n--;
  }

  while (o && n) {
    var new_block = new_blocks[n - 1];
    var old_block = old_blocks[o - 1];
    var new_key = new_block.key;
    var old_key = old_block.key;

    if (new_block === old_block) {
      // do nothing
      next = new_block.first;
      o--;
      n--;
    } else if (!new_lookup[old_key]) {
      // remove old block
      destroy(old_block, lookup);
      o--;
    } else if (!lookup[new_key] || will_move[new_key]) {
      insert(new_block);
    } else if (did_move[old_key]) {
      o--;
    } else if (deltas[new_key] > deltas[old_key]) {
      did_move[new_key] = true;
      insert(new_block);
    } else {
      will_move[old_key] = true;
      o--;
    }
  }

  while (o--) {
    var old_block = old_blocks[o];
    if (!new_lookup[old_block.key]) destroy(old_block, lookup);
  }

  while (n) {
    insert(new_blocks[n - 1]);
  }

  return new_blocks;
}

function blankObject() {
  return Object.create(null);
}

function destroy(detach) {
  this.destroy = noop;
  this.fire('destroy');
  this.set = noop;

  this._fragment.d(detach !== false);

  this._fragment = null;
  this._state = {};
}

function _differs(a, b) {
  return a != a ? b == b : a !== b || a && typeof a === 'object' || typeof a === 'function';
}

function fire(eventName, data) {
  var handlers = eventName in this._handlers && this._handlers[eventName].slice();

  if (!handlers) return;

  for (var i = 0; i < handlers.length; i += 1) {
    var handler = handlers[i];

    if (!handler.__calling) {
      try {
        handler.__calling = true;
        handler.call(this, data);
      } finally {
        handler.__calling = false;
      }
    }
  }
}

function flush(component) {
  component._lock = true;
  callAll(component._beforecreate);
  callAll(component._oncreate);
  callAll(component._aftercreate);
  component._lock = false;
}

function get() {
  return this._state;
}

function init(component, options) {
  component._handlers = blankObject();
  component._slots = blankObject();
  component._bind = options._bind;
  component._staged = {};
  component.options = options;
  component.root = options.root || component;
  component.store = options.store || component.root.store;

  if (!options.root) {
    component._beforecreate = [];
    component._oncreate = [];
    component._aftercreate = [];
  }
}

function on(eventName, handler) {
  var handlers = this._handlers[eventName] || (this._handlers[eventName] = []);
  handlers.push(handler);
  return {
    cancel: function cancel() {
      var index = handlers.indexOf(handler);
      if (~index) handlers.splice(index, 1);
    }
  };
}

function set(newState) {
  this._set(assign({}, newState));

  if (this.root._lock) return;
  flush(this.root);
}

function _set(newState) {
  var oldState = this._state,
      changed = {},
      dirty = false;
  newState = assign(this._staged, newState);
  this._staged = {};

  for (var key in newState) {
    if (this._differs(newState[key], oldState[key])) changed[key] = dirty = true;
  }

  if (!dirty) return;
  this._state = assign(assign({}, oldState), newState);

  this._recompute(changed, this._state);

  if (this._bind) this._bind(changed, this._state);

  if (this._fragment) {
    this.fire("state", {
      changed: changed,
      current: this._state,
      previous: oldState
    });

    this._fragment.p(changed, this._state);

    this.fire("update", {
      changed: changed,
      current: this._state,
      previous: oldState
    });
  }
}

function _stage(newState) {
  assign(this._staged, newState);
}

function callAll(fns) {
  while (fns && fns.length) {
    fns.shift()();
  }
}

function _mount(target, anchor) {
  this._fragment[this._fragment.i ? 'i' : 'm'](target, anchor || null);
}

var proto = {
  destroy: destroy,
  get: get,
  fire: fire,
  on: on,
  set: set,
  _recompute: noop,
  _set: _set,
  _stage: _stage,
  _mount: _mount,
  _differs: _differs
};

var KEY_RETURN = 13;
var KEY_ESCAPE = 27;
var KEY_UP = 38;
var KEY_DOWN = 40;

/* lib/option.svelte generated by Svelte v2.16.1 */

function oncreate() {
  this.sideEffects = manageSideEffects();
}

function ondestroy() {
  this.sideEffects.removeAll();
}

function onupdate(_ref) {
  var _this = this;

  var changed = _ref.changed,
      current = _ref.current,
      previous = _ref.previous;
  var value = current.value,
      decorateOption = current.decorateOption;

  if (changed.content && value !== null) {
    this.sideEffects.removeAll();
    this.sideEffects.add(function () {
      return decorateOption(_this.refs.option);
    });
  }
}

function create_main_fragment(component, ctx) {
  var if_block_anchor;

  function select_block_type(ctx) {
    if (ctx.value !== null) return create_if_block;
    return create_else_block;
  }

  var current_block_type = select_block_type(ctx);
  var if_block = current_block_type(component, ctx);
  return {
    c: function c() {
      if_block.c();
      if_block_anchor = createComment();
    },
    m: function m(target, anchor) {
      if_block.m(target, anchor);
      insert(target, if_block_anchor, anchor);
    },
    p: function p(changed, ctx) {
      if (current_block_type === (current_block_type = select_block_type(ctx)) && if_block) {
        if_block.p(changed, ctx);
      } else {
        if_block.d(1);
        if_block = current_block_type(component, ctx);
        if_block.c();
        if_block.m(if_block_anchor.parentNode, if_block_anchor);
      }
    },
    d: function d(detach) {
      if_block.d(detach);

      if (detach) {
        detachNode(if_block_anchor);
      }
    }
  };
} // (36:0) {:else}


function create_else_block(component, ctx) {
  var li;
  return {
    c: function c() {
      li = createElement("li");
    },
    m: function m(target, anchor) {
      insert(target, li, anchor);
      li.innerHTML = ctx.content;
    },
    p: function p(changed, ctx) {
      if (changed.content) {
        li.innerHTML = ctx.content;
      }
    },
    d: function d(detach) {
      if (detach) {
        detachNode(li);
      }
    }
  };
} // (23:0) {#if value !== null}


function create_if_block(component, ctx) {
  var li, li_id_value, li_class_value, li_aria_selected_value;

  function click_handler(event) {
    component.fire("click", event);
  }

  return {
    c: function c() {
      li = createElement("li");
      addListener(li, "click", click_handler);
      li.id = li_id_value = ctx.namespace + "-item-" + ctx.id + "-" + ctx.index;
      li.className = li_class_value = ctx.namespace + "-item";
      setAttribute(li, "role", "option");
      setAttribute(li, "aria-selected", li_aria_selected_value = ctx.isActive ? 'true' : 'false');
      li.tabIndex = "-1";
      toggleClass(li, "is-selected", ctx.isActive);
    },
    m: function m(target, anchor) {
      insert(target, li, anchor);
      li.innerHTML = ctx.content;
      component.refs.option = li;
    },
    p: function p(changed, ctx) {
      if (changed.content) {
        li.innerHTML = ctx.content;
      }

      if ((changed.namespace || changed.id || changed.index) && li_id_value !== (li_id_value = ctx.namespace + "-item-" + ctx.id + "-" + ctx.index)) {
        li.id = li_id_value;
      }

      if (changed.namespace && li_class_value !== (li_class_value = ctx.namespace + "-item")) {
        li.className = li_class_value;
      }

      if (changed.isActive && li_aria_selected_value !== (li_aria_selected_value = ctx.isActive ? 'true' : 'false')) {
        setAttribute(li, "aria-selected", li_aria_selected_value);
      }

      if (changed.namespace || changed.isActive) {
        toggleClass(li, "is-selected", ctx.isActive);
      }
    },
    d: function d(detach) {
      if (detach) {
        detachNode(li);
      }

      removeListener(li, "click", click_handler);
      if (component.refs.option === li) component.refs.option = null;
    }
  };
}

function Option(options) {
  var _this2 = this;

  init(this, options);
  this.refs = {};
  this._state = assign({}, options.data);
  this._intro = true;
  this._handlers.update = [onupdate];
  this._handlers.destroy = [ondestroy];
  this._fragment = create_main_fragment(this, this._state);

  this.root._oncreate.push(function () {
    oncreate.call(_this2);

    _this2.fire("update", {
      changed: assignTrue({}, _this2._state),
      current: _this2._state
    });
  });

  if (options.target) {
    this._fragment.c();

    this._mount(options.target, options.anchor);

    flush(this);
  }
}

assign(Option.prototype, proto);

function isMouseClick(keycode) {
  return keycode >= 1 && keycode <= 3;
}

function moveCursorToEnd(element) {
  if (typeof element.selectionStart === 'number') {
    element.selectionStart = element.selectionEnd = element.value.length;
  } else if (typeof element.createTextRange !== 'undefined') {
    element.focus();
    var range = element.createTextRange();
    range.collapse(false);
    range.select();
  }
}

function moveCursorToEndSideEffect(element) {
  var id = requestAnimationFrame(function () {
    moveCursorToEnd(element);
  });
  return function () {
    cancelAnimationFrame(id);
  };
}

var count = 0;

function hasResults(_ref) {
  var results = _ref.results;
  return results.length !== 0;
}

function preparedResults(_ref2) {
  var results = _ref2.results;
  return results.map(function (result, index) {
    var id = "" + result.content;

    if (result.value === null) {
      id = "null" + index + id;
    } else {
      id = "" + result.value + id;
    }

    if (typeof result.meta !== 'undefined') {
      id = "" + JSON.stringify(result.meta) + id;
    }

    return _extends({}, result, {
      id: id
    });
  });
}

function data() {
  var id = count;
  count = count + 1;
  return {
    namespace: 'x-Autosuggest',
    value: '',
    fixedValue: '',
    // Original user input value
    isOpened: false,
    loading: false,
    id: id,
    position: null,
    results: [],
    direction: null,
    isComponentActive: true
  };
}

function inputdecorated(node, callback) {
  var _this$get = this.get(),
      decorateInputEvent = _this$get.decorateInputEvent;

  var listener = decorateInputEvent(callback);
  node.addEventListener('input', listener, false);
  return {
    destroy: function destroy() {
      node.removeEventListener('input', listener, false);
    }
  };
}
var methods = {
  setValue: function setValue(value) {
    var _this$get2 = this.get(),
        results = _this$get2.results;

    this.set({
      value: value,
      fixedValue: value,
      position: null,
      isOpened: false,
      results: results.map(function (result, index) {
        return _extends({}, result, {
          active: false
        });
      })
    });
  },
  handlePointerSelect: function handlePointerSelect(event, value, meta, selectedIndex) {
    var _this$get3 = this.get(),
        results = _this$get3.results,
        onOptionSelect = _this$get3.onOptionSelect;

    this.setValue(value);
    onOptionSelect(event, value, meta);
  },
  handleGlobalEvent: function handleGlobalEvent(event) {
    var _this$get4 = this.get(),
        fixedValue = _this$get4.fixedValue;

    var keycode = event.which;
    var target = event.target;

    if (isMouseClick(keycode) && this.refs.container.contains(target)) {
      return;
    }

    this.set({
      value: fixedValue,
      position: null,
      isOpened: false
    });
  },
  handleKeydownEvent: function handleKeydownEvent(event) {
    var keycode = event.which;

    var _this$get5 = this.get(),
        isOpened = _this$get5.isOpened,
        onOptionSelect = _this$get5.onOptionSelect,
        position = _this$get5.position,
        results = _this$get5.results,
        fixedValue = _this$get5.fixedValue;

    switch (keycode) {
      case KEY_UP:
      case KEY_DOWN:
        this.set({
          isOpened: true,
          direction: keycode
        });
        this.navigate();
        break;

      case KEY_ESCAPE:
        // We need to prevent default action for `[type="search"]`
        event.preventDefault();
        this.set({
          value: fixedValue,
          position: null,
          isOpened: false
        });
        break;

      case KEY_RETURN:
        if (isOpened && position !== null) {
          var result = results[position];
          this.setValue(result.value);
          onOptionSelect(event, result.value, result.meta);
        }

        break;
    }
  },
  handleInputEvent: function handleInputEvent(event) {
    var _this$get6 = this.get(),
        value = _this$get6.value;

    this.set({
      fixedValue: value
    });
    this.getData(value);
  },
  handleBlurEvent: function handleBlurEvent(event) {
    var _this$get7 = this.get(),
        fixedValue = _this$get7.fixedValue;

    var target = event.relatedTarget;

    if (target && this.refs.container.contains(target)) {
      return;
    }

    this.set({
      value: fixedValue,
      position: null,
      isOpened: false
    });
  },
  getData: function getData(query) {
    var _this = this;

    var _this$get8 = this.get(),
        onQueryInput = _this$get8.onQueryInput;

    this.set({
      loading: true
    });
    onQueryInput(query).then(function (results) {
      _this.set({
        loading: false,
        position: null,
        isOpened: results.length !== 0,
        results: results.map(function (result) {
          return _extends({}, result, {
            active: false
          });
        })
      });

      return results;
    });
  },
  navigate: function navigate() {
    var _this$get9 = this.get(),
        results = _this$get9.results,
        position = _this$get9.position,
        direction = _this$get9.direction,
        fixedValue = _this$get9.fixedValue,
        value = _this$get9.value;

    var newPosition = position;
    var newValue = value;

    if (results.length === 0) {
      return;
    }

    if (direction === KEY_UP) {
      if (newPosition === null) {
        newPosition = results.length - 1;
      } else if (newPosition === 0) {
        newPosition = null;
      } else {
        newPosition = newPosition - 1;
      }
    } else if (direction === KEY_DOWN) {
      if (newPosition === null) {
        newPosition = 0;
      } else if (newPosition === results.length - 1) {
        newPosition = null;
      } else {
        newPosition = newPosition + 1;
      }
    }

    if (newPosition !== null) {
      var validIndexes = results.map(function (result, index) {
        return result.value !== null ? index : null;
      }).filter(function (value) {
        return value !== null;
      });

      if (direction === KEY_UP) {
        validIndexes.reverse();
      }

      newPosition = validIndexes.filter(function (index) {
        if (direction === KEY_UP) {
          return index <= newPosition;
        }

        return index >= newPosition;
      }).shift();

      if (typeof newPosition !== 'number') {
        newPosition = null;
      }
    }

    if (newPosition === null) {
      newValue = fixedValue;
    } else {
      newValue = results[newPosition].value;
    }

    this.set({
      value: newValue,
      position: newPosition,
      results: results.map(function (result, index) {
        return _extends({}, result, {
          active: index === newPosition
        });
      })
    });
  }
};

function oncreate$1() {
  var _this2 = this;

  this.sideEffects = manageSideEffects();
  this.sideEffects.add(function () {
    var observer = new MutationObserver(function (mutations) {
      mutations.forEach(function (mutation) {
        if (mutation.attributeName === 'disabled') {
          if (mutation.oldValue === null) {
            _this2.set({
              isOpened: false
            });
          }
        }
      });
    });
    observer.observe(_this2.refs.input, {
      attributes: true,
      attributeFilter: ['disabled'],
      attributeOldValue: true
    });
    return function () {
      observer.disconnect();
    };
  });
  this.sideEffects.add(function () {
    return moveCursorToEndSideEffect(_this2.refs.input);
  });
}

function ondestroy$1() {
  this.refs.input.removeAttribute('autocomplete');
  this.sideEffects.removeAll();
}

function onupdate$1(_ref3) {
  var _this3 = this;

  var changed = _ref3.changed,
      current = _ref3.current;

  if (changed.position && current.direction === KEY_UP) {
    this.sideEffects.removeAll();
    this.sideEffects.add(function () {
      return moveCursorToEndSideEffect(_this3.refs.input);
    });
  }
}

function get_each_context(ctx, list, i) {
  var child_ctx = Object.create(ctx);
  child_ctx.result = list[i];
  child_ctx.index = i;
  return child_ctx;
}

function create_main_fragment$1(component, ctx) {
  var text0,
      div,
      input,
      input_updating = false,
      inputdecorated_handler,
      input_autocomplete_value,
      input_role_value,
      input_aria_autocomplete_value,
      input_aria_owns_value,
      input_aria_activedescendant_value,
      text1;

  function onwindowclick(event) {
    component.handleGlobalEvent(event);
  }

  document.addEventListener("click", onwindowclick);

  function input_input_handler() {
    input_updating = true;
    component.set({
      value: input.value
    });
    input_updating = false;
  }

  function keydown_handler(event) {
    component.handleKeydownEvent(event);
  }

  function focusout_handler(event) {
    component.handleBlurEvent(event);
  }

  var if_block = ctx.hasResults && create_if_block$1(component, ctx);
  return {
    c: function c() {
      text0 = createText("\n");
      div = createElement("div");
      input = component.options.elementToHandle;
      text1 = createText("\n\t");
      if (if_block) if_block.c();
      addListener(input, "input", input_input_handler);
      inputdecorated_handler = inputdecorated.call(component, input, function (event) {
        component.handleInputEvent(event);
      });
      addListener(input, "keydown", keydown_handler);
      addListener(input, "focusout", focusout_handler);
      input.autocomplete = input_autocomplete_value = ctx.isComponentActive ? "off" : null;
      setAttribute(input, "role", input_role_value = ctx.isComponentActive ? "combobox" : null);
      setAttribute(input, "aria-autocomplete", input_aria_autocomplete_value = ctx.isComponentActive ? "list" : null);
      setAttribute(input, "aria-owns", input_aria_owns_value = ctx.isComponentActive && ctx.hasResults ? ctx.namespace + "-results-" + ctx.id : null);
      setAttribute(input, "aria-activedescendant", input_aria_activedescendant_value = ctx.isComponentActive ? ctx.position !== null ? ctx.namespace + "-item-" + ctx.id + "-" + ctx.position : null : null);
      toggleClass(input, "x-Autosuggest-input", ctx.isComponentActive);
      div.className = ctx.namespace;
      toggleClass(div, "is-opened", ctx.isOpened);
    },
    m: function m(target, anchor) {
      insert(target, text0, anchor);
      insert(target, div, anchor);
      append(div, input);
      component.refs.input = input;
      input.value = ctx.value;
      append(div, text1);
      if (if_block) if_block.m(div, null);
      component.refs.container = div;
    },
    p: function p(changed, ctx) {
      if (!input_updating && changed.value) input.value = ctx.value;

      if (changed.isComponentActive && input_autocomplete_value !== (input_autocomplete_value = ctx.isComponentActive ? "off" : null)) {
        input.autocomplete = input_autocomplete_value;
      }

      if (changed.isComponentActive && input_role_value !== (input_role_value = ctx.isComponentActive ? "combobox" : null)) {
        setAttribute(input, "role", input_role_value);
      }

      if (changed.isComponentActive && input_aria_autocomplete_value !== (input_aria_autocomplete_value = ctx.isComponentActive ? "list" : null)) {
        setAttribute(input, "aria-autocomplete", input_aria_autocomplete_value);
      }

      if ((changed.isComponentActive || changed.hasResults || changed.namespace || changed.id) && input_aria_owns_value !== (input_aria_owns_value = ctx.isComponentActive && ctx.hasResults ? ctx.namespace + "-results-" + ctx.id : null)) {
        setAttribute(input, "aria-owns", input_aria_owns_value);
      }

      if ((changed.isComponentActive || changed.position || changed.namespace || changed.id) && input_aria_activedescendant_value !== (input_aria_activedescendant_value = ctx.isComponentActive ? ctx.position !== null ? ctx.namespace + "-item-" + ctx.id + "-" + ctx.position : null : null)) {
        setAttribute(input, "aria-activedescendant", input_aria_activedescendant_value);
      }

      if (changed.isComponentActive) {
        toggleClass(input, "x-Autosuggest-input", ctx.isComponentActive);
      }

      if (ctx.hasResults) {
        if (if_block) {
          if_block.p(changed, ctx);
        } else {
          if_block = create_if_block$1(component, ctx);
          if_block.c();
          if_block.m(div, null);
        }
      } else if (if_block) {
        if_block.d(1);
        if_block = null;
      }

      if (changed.namespace) {
        div.className = ctx.namespace;
      }

      if (changed.namespace || changed.isOpened) {
        toggleClass(div, "is-opened", ctx.isOpened);
      }
    },
    d: function d(detach) {
      component.options.target.appendChild(input);
      document.removeEventListener("click", onwindowclick);

      if (detach) {
        detachNode(text0);
        detachNode(div);
      }

      removeListener(input, "input", input_input_handler);
      inputdecorated_handler.destroy();
      removeListener(input, "keydown", keydown_handler);
      removeListener(input, "focusout", focusout_handler);
      if (component.refs.input === input) component.refs.input = null;
      if (if_block) if_block.d();
      if (component.refs.container === div) component.refs.container = null;
    }
  };
} // (333:1) {#if hasResults}


function create_if_block$1(component, ctx) {
  var div,
      ul,
      each_blocks_1 = [],
      each_lookup = blankObject(),
      ul_class_value,
      div_id_value,
      div_aria_expanded_value,
      div_class_value;
  var each_value = ctx.preparedResults;

  var get_key = function get_key(ctx) {
    return ctx.result.id;
  };

  for (var i = 0; i < each_value.length; i += 1) {
    var child_ctx = get_each_context(ctx, each_value, i);
    var key = get_key(child_ctx);
    each_blocks_1[i] = each_lookup[key] = create_each_block(component, key, child_ctx);
  }

  return {
    c: function c() {
      div = createElement("div");
      ul = createElement("ul");

      for (i = 0; i < each_blocks_1.length; i += 1) {
        each_blocks_1[i].c();
      }

      ul.className = ul_class_value = ctx.namespace + "-list";
      div.id = div_id_value = ctx.namespace + "-results-" + ctx.id;
      setAttribute(div, "role", "listbox");
      setAttribute(div, "aria-expanded", div_aria_expanded_value = ctx.isOpened ? 'true' : 'false');
      div.className = div_class_value = ctx.namespace + "-results";
      toggleClass(div, "is-opened", ctx.isOpened);
    },
    m: function m(target, anchor) {
      insert(target, div, anchor);
      append(div, ul);

      for (i = 0; i < each_blocks_1.length; i += 1) {
        each_blocks_1[i].m(ul, null);
      }
    },
    p: function p(changed, ctx) {
      var each_value = ctx.preparedResults;
      each_blocks_1 = updateKeyedEach(each_blocks_1, component, changed, get_key, 1, ctx, each_value, each_lookup, ul, destroyBlock, create_each_block, "m", null, get_each_context);

      if (changed.namespace && ul_class_value !== (ul_class_value = ctx.namespace + "-list")) {
        ul.className = ul_class_value;
      }

      if ((changed.namespace || changed.id) && div_id_value !== (div_id_value = ctx.namespace + "-results-" + ctx.id)) {
        div.id = div_id_value;
      }

      if (changed.isOpened && div_aria_expanded_value !== (div_aria_expanded_value = ctx.isOpened ? 'true' : 'false')) {
        setAttribute(div, "aria-expanded", div_aria_expanded_value);
      }

      if (changed.namespace && div_class_value !== (div_class_value = ctx.namespace + "-results")) {
        div.className = div_class_value;
      }

      if (changed.namespace || changed.isOpened) {
        toggleClass(div, "is-opened", ctx.isOpened);
      }
    },
    d: function d(detach) {
      if (detach) {
        detachNode(div);
      }

      for (i = 0; i < each_blocks_1.length; i += 1) {
        each_blocks_1[i].d();
      }
    }
  };
} // (342:3) {#each preparedResults as result, index (result.id)}


function create_each_block(component, key_1, ctx) {
  var first;
  var option_initial_data = {
    namespace: ctx.namespace,
    id: ctx.id,
    index: ctx.index,
    content: ctx.result.content,
    isActive: ctx.result.active,
    value: ctx.result.value,
    decorateOption: ctx.decorateOption
  };
  var option = new Option({
    root: component.root,
    store: component.store,
    data: option_initial_data
  });
  option.on("click", function (event) {
    component.handlePointerSelect(event, ctx.result.value, ctx.result.meta, ctx.index);
  });
  return {
    key: key_1,
    first: null,
    c: function c() {
      first = createComment();

      option._fragment.c();

      this.first = first;
    },
    m: function m(target, anchor) {
      insert(target, first, anchor);

      option._mount(target, anchor);
    },
    p: function p(changed, _ctx) {
      ctx = _ctx;
      var option_changes = {};
      if (changed.namespace) option_changes.namespace = ctx.namespace;
      if (changed.id) option_changes.id = ctx.id;
      if (changed.preparedResults) option_changes.index = ctx.index;
      if (changed.preparedResults) option_changes.content = ctx.result.content;
      if (changed.preparedResults) option_changes.isActive = ctx.result.active;
      if (changed.preparedResults) option_changes.value = ctx.result.value;
      if (changed.decorateOption) option_changes.decorateOption = ctx.decorateOption;

      option._set(option_changes);
    },
    d: function d(detach) {
      if (detach) {
        detachNode(first);
      }

      option.destroy(detach);
    }
  };
}

function Index(options) {
  var _this4 = this;

  init(this, options);
  this.refs = {};
  this._state = assign(data(), options.data);

  this._recompute({
    results: 1
  }, this._state);

  this._intro = true;
  this._handlers.update = [onupdate$1];
  this._handlers.destroy = [ondestroy$1];
  this._fragment = create_main_fragment$1(this, this._state);

  this.root._oncreate.push(function () {
    oncreate$1.call(_this4);

    _this4.fire("update", {
      changed: assignTrue({}, _this4._state),
      current: _this4._state
    });
  });

  if (options.target) {
    this._fragment.c();

    this._mount(options.target, options.anchor);

    flush(this);
  }
}

assign(Index.prototype, proto);
assign(Index.prototype, methods);

Index.prototype._recompute = function _recompute(changed, state) {
  if (changed.results) {
    if (this._differs(state.hasResults, state.hasResults = hasResults(state))) changed.hasResults = true;
    if (this._differs(state.preparedResults, state.preparedResults = preparedResults(state))) changed.preparedResults = true;
  }
};

var index = (function (element, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$decorateOpti = _options.decorateOption,
      decorateOption = _options$decorateOpti === void 0 ? function (node) {} : _options$decorateOpti,
      _options$decorateInpu = _options.decorateInputEvent,
      decorateInputEvent = _options$decorateInpu === void 0 ? function (listener) {
    return listener;
  } : _options$decorateInpu,
      _options$onOptionSele = _options.onOptionSelect,
      onOptionSelect = _options$onOptionSele === void 0 ? function (event, value, meta) {} : _options$onOptionSele,
      _options$onQueryInput = _options.onQueryInput,
      onQueryInput = _options$onQueryInput === void 0 ? function (query) {
    return Promise.resolve([]);
  } : _options$onQueryInput;
  var value = element.value;
  var instance = new Index({
    target: element.parentElement,
    elementToHandle: element,
    data: {
      value: value,
      fixedValue: value,
      decorateOption: decorateOption,
      decorateInputEvent: decorateInputEvent,
      onOptionSelect: onOptionSelect,
      onQueryInput: onQueryInput
    }
  });
  return {
    destroy: function destroy() {
      instance.set({
        isComponentActive: false
      });
      instance.destroy();
    }
  };
});

module.exports = index;
//# sourceMappingURL=index.cjs.js.map
